/***********************************************************
 * $Id: FieldInfo.java 299 2014-01-07 21:59:51Z remoting $
 * 
 * remoting framework of the clazzes project.
 * http://www.clazzes.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Created: 02.02.2011
 * 
 ***********************************************************/

package org.clazzes.remoting.maven;

import java.lang.reflect.Field;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.lang.reflect.WildcardType;

/**
 * Information on a class field, which is possibly retrieved from
 * the java source.
 */
public class FieldInfo {

	private final Field field;
	private final String   javaDoc;
	
	public FieldInfo(Field field) {
		this.field = field;
		this.javaDoc = null;
	}
	
	public FieldInfo(Field field, String javaDoc) {
		this.field = field;
		this.javaDoc = javaDoc;
	}
	
	public String getName() {
		return this.field.getName();
	}
		
	public int getModifiers() {
		return this.field.getModifiers();
	}

	public Class<?> getType() {
		return this.field.getType();
	}
	
	public int getGenericParameterCount() {
		
		Type gt = this.field.getGenericType();
		
		if (gt instanceof ParameterizedType) {
			
			ParameterizedType pt = (ParameterizedType)gt;
			return pt.getActualTypeArguments().length;
		}
		else {
			return 0;
		}
	}
	
	public Class<?> getGenericParameter(int i) {
		
		Type gt = this.field.getGenericType();
		
		if (gt instanceof ParameterizedType) {
			
			ParameterizedType pt = (ParameterizedType)gt;
		
			Type typeArgument = pt.getActualTypeArguments()[i];
			
			if (typeArgument instanceof WildcardType) {
				
				WildcardType wt = (WildcardType)typeArgument;
				
				Type[] bounds = wt.getUpperBounds();
				
				if (bounds.length == 1) {
					return (Class<?>)bounds[0];
				}
				
			}
			else if (typeArgument instanceof Class) {
				return (Class<?>) typeArgument;
			}
		}
		
		throw new IllegalArgumentException("Field ["+this.getName()+"] is not parametrized and has not a single type argument at position ["+i+"].");
	}
		
	public String getJavaDoc() {
		return this.javaDoc;
	}

	public Field getField() {
		
		return this.field;
	}
}
